#
# Copyright (c) 2015 LAAS/CNRS
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software for any purpose
# with or without   fee is hereby granted, provided   that the above  copyright
# notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS  SOFTWARE INCLUDING ALL  IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS. IN NO EVENT SHALL THE AUTHOR  BE LIABLE FOR ANY SPECIAL, DIRECT,
# INDIRECT, OR CONSEQUENTIAL DAMAGES OR  ANY DAMAGES WHATSOEVER RESULTING  FROM
# LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
# OTHER TORTIOUS ACTION,   ARISING OUT OF OR IN    CONNECTION WITH THE USE   OR
# PERFORMANCE OF THIS SOFTWARE.
#
#                                             Anthony Mallet on Mon May 18 2015
#
from __future__ import print_function

import datetime
import getopt
import os
import sys

import node
import server
import session
from util import *
from version import __version__


# --- usage ----------------------------------------------------------------
#
def usage():
  return 'usage: %s [-p port] [-v] [--version]' % argv0()


# --- main -----------------------------------------------------------------
#
def main():
  # init
  port = 8080

  # process options
  try:
    opts, args = getopt.getopt(sys.argv[1:],
                               'p:vh', ['port', 'version', 'help'])
  except getopt.GetoptError as err:
    print(err, file=sys.stderr)
    print(usage(), file=sys.stderr)
    sys.exit(2)

  for o, a in opts:
    if o in ('-p', '--port'):
      try:
        port = int(a)
      except:
        warnx('invalid port number -- %s', a)
        sys.exit(2)
    elif o == '-v':
      verbose()
    elif o == '--version':
      print(__version__)
      sys.exit()
    elif o in ('-h', '--help'):
      print(usage())
      sys.exit()
    else:
      assert False, "unhandled option"

  # process extraneous arguments
  if len(args) > 0:
    print('too many arguments -- %s' % ' '.join(args), file=sys.stderr)
    print(usage(), file=sys.stderr)
    sys.exit(2)

  # start server
  node.init()
  server.start(port)
  infox('accepting connections on port %d' % port)

  # loop forever
  node.run()

  # clean up
  infox('terminating on shutdown request')
  node.fini()
  session.kill()
  server.stop()
